@@ -1,3 +1,4 @@
+import TaskOptionExtractor.generateZshTaskOpts
 import org.gradle.api.DefaultTask
 import org.gradle.api.file.RegularFileProperty
 import org.gradle.api.tasks.CacheableTask
@@ -17,6 +18,7 @@ import org.gradle.internal.buildoption.EnumBuildOption
 import org.gradle.internal.buildoption.IntegerBuildOption
 import org.gradle.internal.buildoption.ListBuildOption
 import org.gradle.internal.buildoption.StringBuildOption
+import org.gradle.internal.extensions.stdlib.uncheckedCast
 import org.gradle.internal.logging.LoggingConfigurationBuildOptions
 import org.gradle.launcher.cli.converter.WelcomeMessageBuildOptions
 import org.gradle.launcher.daemon.configuration.DaemonBuildOptions
@@ -58,28 +60,56 @@ abstract class GenerateCompletionScriptsTask : DefaultTask() {
     fun generate() {
         logger.lifecycle("Starting generation of completion scripts...")
 
-        // STEP 1: Get the command line options from Gradle's internal API
         val allOptions = collectAllBuildOptions()
         logger.lifecycle("Successfully extracted start parameter from Gradle instance.")
 
         val allCliOptions = extractCliOptions(allOptions)
         logger.lifecycle("Successfully extracted ${allCliOptions.size} CLI options from Gradle API.")
 
-        // Add help and version options that aren't exposed via the API
-        allCliOptions += createHelpOption()
-        allCliOptions += createVersionOption()
+        allCliOptions += listOf(
+            createHelpOption(),
+            createVersionOption(),
+            createShowVersionOption(),
+            createRerunOption()
+        )
 
         val allPropertyNames = extractPropertyOptions(allOptions)
 
-        // STEP 2: Format the extracted options into Bash and Zsh specific strings
+        val wrapperOptions = extractWrapperOptions()
+        logger.lifecycle("Successfully extracted ${wrapperOptions.size} wrapper task options.")
+
+        val testOptions = extractTestOptions()
+        logger.lifecycle("Successfully extracted ${testOptions.size} test task options.")
+
+        val initOptions = extractInitOptions()
+        logger.lifecycle("Successfully extracted ${initOptions.size} init task options.")
+
+        val dependenciesOptions = extractDependenciesOptions()
+        logger.lifecycle("Successfully extracted ${dependenciesOptions.size} dependencies task options.")
+
+        val dependencyInsightOptions = extractDependencyInsightOptions()
+        logger.lifecycle("Successfully extracted ${dependencyInsightOptions.size} dependencyInsight task options.")
+
+        val tasksOptions = extractTasksOptions()
+        logger.lifecycle("Successfully extracted ${tasksOptions.size} tasks task options.")
+
+        val helpOptions = extractHelpOptions()
+        logger.lifecycle("Successfully extracted ${helpOptions.size} help task options.")
+
         val bashLongOpts = generateBashLongOpts(allCliOptions)
         val bashShortOpts = getBashShortOpts(allCliOptions)
         allCliOptions += allPropertyNames
         val zshMainOpts = generateZshMainOpts(allCliOptions)
         val zshSubcommandOpts = generateZshSubcommandOpts(allCliOptions)
         val properties = generatePropertiesOpts(allPropertyNames)
+        val zshWrapperOpts = generateZshTaskOpts(wrapperOptions)
+        val zshTestOpts = generateZshTaskOpts(testOptions) + " \\\n                '(-)*:: :->task-or-option'"
+        val zshInitOpts = generateZshTaskOpts(initOptions)
+        val zshDependenciesOpts = generateZshTaskOpts(dependenciesOptions)
+        val zshDependencyInsightOpts = generateZshTaskOpts(dependencyInsightOptions)
+        val zshTasksOpts = generateZshTaskOpts(tasksOptions)
+        val zshHelpOpts = generateZshTaskOpts(helpOptions)
 
-        // STEP 3: Read templates and substitute placeholders
         val bashCompletionScript = bashTemplate.asFile.get().readText()
             .replace("{{GENERATED_BASH_LONG_OPTIONS}}", bashLongOpts.trimEnd())
             .replace("{{GENERATED_BASH_SHORT_OPTIONS}}", bashShortOpts.trimEnd())
@@ -88,8 +118,14 @@ abstract class GenerateCompletionScriptsTask : DefaultTask() {
         val zshCompletionScript = zshTemplate.asFile.get().readText()
             .replace("{{GENERATED_ZSH_MAIN_OPTIONS}}", zshMainOpts)
             .replace("{{GENERATED_ZSH_SUBCOMMAND_OPTIONS}}", zshSubcommandOpts)
+            .replace("{{GENERATED_ZSH_WRAPPER_OPTIONS}}", zshWrapperOpts)
+            .replace("{{GENERATED_ZSH_TEST_OPTIONS}}", zshTestOpts)
+            .replace("{{GENERATED_ZSH_INIT_OPTIONS}}", zshInitOpts)
+            .replace("{{GENERATED_ZSH_DEPENDENCIES_OPTIONS}}", zshDependenciesOpts)
+            .replace("{{GENERATED_ZSH_DEPENDENCY_INSIGHT_OPTIONS}}", zshDependencyInsightOpts)
+            .replace("{{GENERATED_ZSH_TASKS_OPTIONS}}", zshTasksOpts)
+            .replace("{{GENERATED_ZSH_HELP_OPTIONS}}", zshHelpOpts)
 
-        // STEP 4: Write generated files
         bashOutputFile.asFile.get().writeText(bashCompletionScript)
         zshOutputFile.asFile.get().writeText(zshCompletionScript)
 
@@ -99,8 +135,8 @@ abstract class GenerateCompletionScriptsTask : DefaultTask() {
         logger.lifecycle("\nGeneration complete!")
     }
 
-    private fun collectAllBuildOptions(): List<BuildOption<*>> {
-        return listOf(
+    private fun collectAllBuildOptions() =
+        listOf(
             BuildLayoutParametersBuildOptions(),
             StartParameterBuildOptions(),
             LoggingConfigurationBuildOptions(),
@@ -109,14 +145,10 @@ abstract class GenerateCompletionScriptsTask : DefaultTask() {
             ParallelismBuildOptions(),
             ToolchainBuildOptions.forToolChainConfiguration()
         ).flatMap { it.allOptions }
-    }
 
     private fun extractCliOptions(allOptions: List<BuildOption<*>>): MutableList<CliOption> {
         return allOptions.map { option ->
-            val field = findDeclaredField(option, "commandLineOptionConfigurations")
-            val configurations = field?.get(option) as List<*>
-            val configs = configurations.map { it as CommandLineOptionConfiguration }
-            option to configs
+            option to getCliConfigsFrom(option)
         }
             .filter { it.second.isNotEmpty() }
             .flatMap { (option, configurations) ->
@@ -132,11 +164,7 @@ abstract class GenerateCompletionScriptsTask : DefaultTask() {
                             description = optionDetails.description.replace("\"", "'"),
                             incubating = optionDetails.isIncubating,
                             multipleOccurrencePossible = option is ListBuildOption<*>,
-                            argumentExpected = option is StringBuildOption ||
-                                    option is IntegerBuildOption ||
-                                    option is EnumBuildOption<*, *> ||
-                                    isDirectory ||
-                                    filePattern != null,
+                            argumentExpected = isArgumentExpected(option, isDirectory, filePattern),
                             possibleValues = getPossibleValues(option, optionDetails.longOption),
                             isDirectory = isDirectory,
                             filePattern = filePattern
@@ -168,13 +196,21 @@ abstract class GenerateCompletionScriptsTask : DefaultTask() {
             }.toMutableList()
     }
 
+    private fun isArgumentExpected(
+        option: BuildOption<*>,
+        isDirectory: Boolean,
+        filePattern: String?
+    ): Boolean = option is StringBuildOption ||
+            option is IntegerBuildOption ||
+            option is EnumBuildOption<*, *> ||
+            isDirectory ||
+            filePattern != null
+
     private fun extractPropertyOptions(allOptions: List<BuildOption<*>>): List<CliOption> {
         return allOptions
             .filter { it.property != null && !it.property?.contains(".internal.")!! }
             .mapNotNull { option ->
-                val field = findDeclaredField(option, "commandLineOptionConfigurations")
-                val configurations = field?.get(option) as List<*>
-                val cliConfigs = configurations.map { it as CommandLineOptionConfiguration }
+                val cliConfigs = getCliConfigsFrom(option)
 
                 option.property?.let { propName ->
                     val shortOption = "D$propName="
@@ -194,41 +230,58 @@ abstract class GenerateCompletionScriptsTask : DefaultTask() {
             }
     }
 
-    private fun createHelpOption(): CliOption {
-        return CliOption(
+    private fun getCliConfigsFrom(option: BuildOption<*>) =
+        (getDeclaredField(option, "commandLineOptionConfigurations")
+            .get(option) as List<*>)
+            .map { it as CommandLineOptionConfiguration }
+
+    private fun createHelpOption() =
+        CliOption(
             twoDashOption = "help",
             oneDashOption = "h",
             description = "Shows a help message."
         )
-    }
 
-    private fun createVersionOption(): CliOption {
-        return CliOption(
+    private fun createVersionOption() =
+        CliOption(
             twoDashOption = "version",
             oneDashOption = "v",
-            description = "Shows the version info."
+            description = "Print version info and exit."
         )
-    }
 
-    private fun generateBashLongOpts(options: List<CliOption>): String = options.filter { it.twoDashOption != null }
-        .sortedBy { it.twoDashOption }
-        .joinToString("\n") {
-            it.getSuggestionLine("--${it.twoDashOption}")
-        }
+    private fun createShowVersionOption() =
+        CliOption(
+            twoDashOption = "show-version",
+            oneDashOption = "V",
+            description = "Print version info and continue."
+        )
+
+    private fun createRerunOption() =
+        CliOption(
+            twoDashOption = "rerun",
+            description = "Causes the task to be re-run even if up-to-date."
+        )
+
+    private fun generateBashLongOpts(options: List<CliOption>) =
+        options.filter { it.twoDashOption != null }
+            .sortedBy { it.twoDashOption }
+            .joinToString("\n") {
+                it.getSuggestionLine("--${it.twoDashOption}")
+            }
 
 
-    private fun getBashShortOpts(options: List<CliOption>): String = options.filter { it.oneDashOption != null }
-        .sortedBy { it.oneDashOption?.lowercase(Locale.getDefault()) }
-        .joinToString("\n") {
-            it.getSuggestionLine("-${it.oneDashOption}")
-        }
+    private fun getBashShortOpts(options: List<CliOption>) =
+        options.filter { it.oneDashOption != null }
+            .sortedBy { it.oneDashOption?.lowercase(Locale.getDefault()) }
+            .joinToString("\n") {
+                it.getSuggestionLine("-${it.oneDashOption}")
+            }
 
-    private fun generateZshMainOpts(options: List<CliOption>): String {
-        return options.sortedBy { it.twoDashOption ?: it.oneDashOption }
+    private fun generateZshMainOpts(options: List<CliOption>) =
+        options.sortedBy { it.twoDashOption ?: it.oneDashOption }
             .joinToString(" \\\n        ") { option ->
                 option.getOptionLine(includeArgumentExpected = true)
             }
-    }
 
     private fun generateZshSubcommandOpts(options: List<CliOption>): String {
         val standaloneOptions = setOf("help", "h", "version", "v", "status", "stop", "gui")
@@ -243,16 +296,60 @@ abstract class GenerateCompletionScriptsTask : DefaultTask() {
             }
     }
 
-    private fun generatePropertiesOpts(allPropertyNames: List<CliOption>): String {
-        return allPropertyNames
+    private fun generatePropertiesOpts(allPropertyNames: List<CliOption>) =
+        allPropertyNames
             .filter { it.oneDashOption != null }
             .joinToString("\n") {
-            val paddingOption = "-${it.oneDashOption!!.padEnd(40)}"
-            "$paddingOption - ${it.description?.lineSequence()?.first() ?: ""}"
-        }
+                val paddingOption = "-${it.oneDashOption!!.padEnd(40)}"
+                "$paddingOption - ${it.description?.lineSequence()?.first() ?: ""}"
+            }
+
+    private fun extractWrapperOptions(): List<TaskOptionDescriptor> {
+        return TaskOptionExtractor.extractTaskOptions("org.gradle.api.tasks.wrapper.Wrapper")
     }
 
-    // Companion object for helper functions and constants
+    private fun extractTestOptions() =
+        TaskOptionExtractor.extractTaskOptions(
+            "org.gradle.api.tasks.testing.Test",
+            "org.gradle.api.tasks.testing.AbstractTestTask"
+        )
+
+    private fun extractInitOptions() =
+        TaskOptionExtractor.extractTaskOptions("org.gradle.buildinit.tasks.InitBuild")
+
+    private fun extractDependenciesOptions(): List<TaskOptionDescriptor> {
+        val options = TaskOptionExtractor.extractTaskOptions(
+            "org.gradle.api.tasks.diagnostics.DependencyReportTask",
+            "org.gradle.api.tasks.diagnostics.AbstractDependencyReportTask"
+        )
+
+        return addCustomCompletionFunctionForConfiguration(options)
+    }
+
+    private fun addCustomCompletionFunctionForConfiguration(options: List<TaskOptionDescriptor>) =
+        options.map { option ->
+            when (option.optionName) {
+                "configuration" -> option.copy(
+                    completionFunction = ":dependency configuration:_gradle_dependency_configurations"
+                )
+
+                else -> option
+            }
+        }
+
+    private fun extractDependencyInsightOptions(): List<TaskOptionDescriptor> {
+        val options =
+            TaskOptionExtractor.extractTaskOptions("org.gradle.api.tasks.diagnostics.DependencyInsightReportTask")
+
+        return addCustomCompletionFunctionForConfiguration(options)
+    }
+
+    private fun extractTasksOptions() =
+        TaskOptionExtractor.extractTaskOptions("org.gradle.api.tasks.diagnostics.TaskReportTask")
+
+    private fun extractHelpOptions() =
+        TaskOptionExtractor.extractTaskOptions("org.gradle.configuration.Help")
+
     companion object {
         private val CACHING_VALUES = listOf("true", "false")
         private val CC_CACHE_PROBLEMS = listOf("fail", "warn")
@@ -294,7 +391,7 @@ abstract class GenerateCompletionScriptsTask : DefaultTask() {
             "init-script" to "*.gradle(|.kts)"
         )
 
-        fun findDeclaredField(obj: Any, fieldName: String): Field? {
+        fun getDeclaredField(obj: Any, fieldName: String): Field {
             var currentClass: Class<*>? = obj.javaClass
 
             while (currentClass != null) {
@@ -302,49 +399,40 @@ abstract class GenerateCompletionScriptsTask : DefaultTask() {
                     return currentClass.getDeclaredField(fieldName).apply {
                         isAccessible = true
                     }
-                } catch (e: NoSuchFieldException) {
+                } catch (_: NoSuchFieldException) {
                     currentClass = currentClass.superclass
                 }
             }
-            return null
+            throw IllegalStateException("Could not find field $fieldName on object $obj")
         }
 
-        fun getPossibleValues(option: BuildOption<*>, longOptionName: String?): List<String> {
-            longOptionName?.let { optName ->
-                HARDCODED_POSSIBLE_VALUES[optName]?.let { return it }
+        fun getPossibleValues(option: BuildOption<*>, longOptionName: String?) =
+            longOptionName?.let { HARDCODED_POSSIBLE_VALUES[it] } ?: getPossibleValuesList(option)
+
+        fun isDirectoryOption(longOptionName: String?) =
+            longOptionName?.let { it in DIRECTORY_OPTIONS } ?: false
+
+        fun getFilePattern(longOptionName: String?) =
+            longOptionName?.let { FILE_OPTIONS[it] }
+
+        fun getPossibleValuesList(option: BuildOption<*>) =
+            when (option) {
+                is EnumBuildOption<*, *> -> getDeclaredField(option, "possibleValues")
+                    .get(option)
+                    ?.uncheckedCast<List<Any>>()
+                    ?.map { it.toString().lowercase() }
+                    ?: emptyList()
+
+                else -> emptyList()
             }
-            return getPossibleValuesList(option)
-        }
 
-        fun isDirectoryOption(longOptionName: String?): Boolean {
-            return longOptionName?.let { it in DIRECTORY_OPTIONS } ?: false
-        }
+        fun getPossibleValuesForProperty(option: BuildOption<*>, propertyName: String) =
+            HARDCODED_POSSIBLE_VALUES[propertyName] ?: getPossibleValuesList(option)
 
-        fun getFilePattern(longOptionName: String?): String? {
-            return longOptionName?.let { FILE_OPTIONS[it] }
-        }
+        fun isDirectoryProperty(propertyName: String) =
+            propertyName in DIRECTORY_OPTIONS
 
-        fun getPossibleValuesList(option: BuildOption<*>): List<String> {
-            return if (option is EnumBuildOption<*, *>) {
-                val valuesField = findDeclaredField(option, "possibleValues")
-                @Suppress("UNCHECKED_CAST")
-                (valuesField?.get(option) as? List<Any>)?.map { it.toString().lowercase() } ?: listOf()
-            } else {
-                listOf()
-            }
-        }
-
-        fun getPossibleValuesForProperty(option: BuildOption<*>, propertyName: String): List<String> {
-            HARDCODED_POSSIBLE_VALUES[propertyName]?.let { return it }
-            return getPossibleValuesList(option)
-        }
-
-        fun isDirectoryProperty(propertyName: String): Boolean {
-            return propertyName in DIRECTORY_OPTIONS
-        }
-
-        fun getFilePatternForProperty(propertyName: String): String? {
-            return FILE_OPTIONS[propertyName]
-        }
+        fun getFilePatternForProperty(propertyName: String) =
+            FILE_OPTIONS[propertyName]
     }
 }
